@@ -18,7 +18,7 @@
 //!
 //! ```no_run
 //! // Move a pin out of the Pins struct and convert to a DynPin
-//! # use rp2040_hal::{pac, gpio::{DynPin, bank0::Gpio12, Pins}, sio::Sio};
+//! # use rp2040_hal::{pac, gpio::{DynPin, bank0::Gpio12, Pins}, Sio};
 //! # let mut peripherals = pac::Peripherals::take().unwrap();
 //! # let sio = Sio::new(peripherals.SIO);
 //! # let pins = Pins::new(peripherals.IO_BANK0,peripherals.PADS_BANK0,sio.gpio_bank0, &mut peripherals.RESETS);
@@ -30,7 +30,7 @@
 //! API.
 //!
 //! ```no_run
-//! # use rp2040_hal::{pac, gpio::{DynPin, Pins}, sio::Sio};
+//! # use rp2040_hal::{pac, gpio::{DynPin, Pins}, Sio};
 //! # let mut peripherals = pac::Peripherals::take().unwrap();
 //! # let sio = Sio::new(peripherals.SIO);
 //! # let pins = Pins::new(peripherals.IO_BANK0,peripherals.PADS_BANK0,sio.gpio_bank0, &mut peripherals.RESETS);
@@ -54,7 +54,7 @@
 //!
 //! ```no_run
 //! # use core::convert::TryInto;
-//! # use rp2040_hal::{pac, gpio::{DynPin, bank0::Gpio12, Pin, Pins, FloatingInput}, sio::Sio};
+//! # use rp2040_hal::{pac, gpio::{DynPin, bank0::Gpio12, Pin, Pins, FloatingInput}, Sio};
 //! # let mut peripherals = pac::Peripherals::take().unwrap();
 //! # let sio = Sio::new(peripherals.SIO);
 //! # let pins = Pins::new(peripherals.IO_BANK0,peripherals.PADS_BANK0,sio.gpio_bank0, &mut peripherals.RESETS);
@@ -78,6 +78,8 @@ use super::pin::{Pin, PinId, PinMode, ValidPinMode};
 use super::reg::RegisterInterface;
 use core::convert::TryFrom;
 
+#[cfg(feature = "eh1_0_alpha")]
+use eh1_0_alpha::digital::blocking as eh1;
 use hal::digital::v2::{InputPin, OutputPin, StatefulOutputPin, ToggleableOutputPin};
 
 //==============================================================================
@@ -91,6 +93,7 @@ pub enum DynDisabled {
     Floating,
     PullDown,
     PullUp,
+    BusKeep,
 }
 
 /// Value-level `enum` for input configurations
@@ -100,6 +103,7 @@ pub enum DynInput {
     Floating,
     PullDown,
     PullUp,
+    BusKeep,
 }
 
 /// Value-level `enum` for output configurations
@@ -539,3 +543,50 @@ impl StatefulOutputPin for DynPin {
         self._is_set_low()
     }
 }
+
+#[cfg(feature = "eh1_0_alpha")]
+impl eh1::OutputPin for DynPin {
+    type Error = Error;
+    #[inline]
+    fn set_high(&mut self) -> Result<(), Self::Error> {
+        self._set_high()
+    }
+    #[inline]
+    fn set_low(&mut self) -> Result<(), Self::Error> {
+        self._set_low()
+    }
+}
+
+#[cfg(feature = "eh1_0_alpha")]
+impl eh1::InputPin for DynPin {
+    type Error = Error;
+    #[inline]
+    fn is_high(&self) -> Result<bool, Self::Error> {
+        self._is_high()
+    }
+    #[inline]
+    fn is_low(&self) -> Result<bool, Self::Error> {
+        self._is_low()
+    }
+}
+
+#[cfg(feature = "eh1_0_alpha")]
+impl eh1::ToggleableOutputPin for DynPin {
+    type Error = Error;
+    #[inline]
+    fn toggle(&mut self) -> Result<(), Self::Error> {
+        self._toggle()
+    }
+}
+
+#[cfg(feature = "eh1_0_alpha")]
+impl eh1::StatefulOutputPin for DynPin {
+    #[inline]
+    fn is_set_high(&self) -> Result<bool, Self::Error> {
+        self._is_set_high()
+    }
+    #[inline]
+    fn is_set_low(&self) -> Result<bool, Self::Error> {
+        self._is_set_low()
+    }
+}
